#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var child_process_1 = require("child_process");
var tmp_1 = require("tmp");
var fs_1 = require("fs");
var inquirer = require("inquirer");
var path = require("path");
var yargsParser = require("yargs-parser");
var presetOptions = [
    {
        value: 'empty',
        name: 'empty             [an empty workspace]'
    },
    {
        value: 'angular',
        name: 'angular           [a workspace with a single Angular application]'
    },
    {
        value: 'angular-ivy',
        name: 'angular-ivy       [a workspace with a single Angular application built using Ivy]'
    },
    {
        value: 'react',
        name: 'react             [a workspace with a single React application]'
    },
    {
        value: 'web-components',
        name: 'web components    [a workspace with a single app built using web components]'
    },
    {
        value: 'full-stack',
        name: 'full-stack        [a workspace with a full stack application (NestJS + Angular Ivy)]'
    }
];
var tsVersion = '~3.4.5';
var cliVersion = '8.4.0-alpha.1';
var nxVersion = '8.4.0-alpha.1';
var angularCliVersion = '8.0.1';
// const tsVersion = '~3.4.5';
// const cliVersion = '/home/vsavkin/projects/products/nx/build/packages/tao';
// const nxVersion = '/home/vsavkin/projects/products/nx/build/packages/workspace';
// const angularCliVersion = '8.0.1';
var parsedArgs = yargsParser(process.argv, {
    string: ['directory', 'cli', 'preset'],
    boolean: ['help']
});
if (parsedArgs.help) {
    showHelp();
    process.exit(0);
}
validateInput(parsedArgs);
var packageManager = determinePackageManager();
determinePreset(parsedArgs).then(function (preset) {
    return determineCli(preset, parsedArgs).then(function (cli) {
        var tmpDir = createSandbox(packageManager, cli);
        createApp(tmpDir, cli, parsedArgs, preset);
        showNxWarning();
        showCliWarning(preset, parsedArgs);
    });
});
function showHelp() {
    console.log("\n  Usage: create-nx-workspace <directory> [options] [new workspace options]\n\n  Create a new Nx workspace\n\n  Options:\n\n    directory                 path to the workspace root directory\n\n    preset                    What to create in a new workspace (options: " + presetOptions
        .map(function (o) { return '"' + o.value + '"'; })
        .join(', ') + ")\n\n    cli                       CLI to power the Nx workspace (options: \"nx\", \"angular\")\n\n    [new workspace options]   any 'new workspace' options\n");
}
function determinePackageManager() {
    // If you have Angular CLI installed, read Angular CLI config.
    // If it isn't not installed, default to 'yarn'.
    var packageManager;
    try {
        packageManager = child_process_1.execSync('ng config -g cli.packageManager', {
            stdio: ['ignore', 'pipe', 'ignore']
        })
            .toString()
            .trim();
    }
    catch (e) {
        packageManager = 'yarn';
    }
    try {
        child_process_1.execSync(packageManager + " --version", {
            stdio: ['ignore', 'ignore', 'ignore']
        });
    }
    catch (e) {
        packageManager = 'npm';
    }
    return packageManager;
}
function validateInput(parsedArgs) {
    var projectName = parsedArgs._[2];
    // check that the workspace name is passed in
    if (!projectName) {
        console.error('Please provide a project name (e.g., create-nx-workspace nrwl-proj)');
        process.exit(1);
    }
    return projectName;
}
function determinePreset(parsedArgs) {
    if (parsedArgs.preset) {
        if (presetOptions.map(function (o) { return o.value; }).indexOf(parsedArgs.preset) === -1) {
            console.error("Invalid preset. It must be one of the following: " + presetOptions
                .map(function (o) { return '"' + o.value + '"'; })
                .join(', ') + ".");
            process.exit(1);
        }
        else {
            return Promise.resolve(parsedArgs.preset);
        }
    }
    else {
        return inquirer
            .prompt([
            {
                name: 'Preset',
                message: "What to create in the new workspace (You can create other applications and libraries at any point using the generate command)",
                default: 'empty',
                type: 'list',
                choices: presetOptions
            }
        ])
            .then(function (a) { return a.Preset; });
    }
}
function determineCli(preset, parsedArgs) {
    if (parsedArgs.cli) {
        if (['nx', 'angular'].indexOf(parsedArgs.cli) === -1) {
            console.error("Invalid cli. It must be one of the following: \"nx\", \"angular\".");
            process.exit(1);
        }
        return Promise.resolve(parsedArgs.cli);
    }
    var angular = {
        package: '@angular/cli',
        version: angularCliVersion,
        command: 'ng'
    };
    var nx = {
        package: '@nrwl/tao',
        version: cliVersion,
        command: 'tao'
    };
    if (preset == 'angular' ||
        preset == 'angular-ivy' ||
        preset == 'full-stack') {
        return Promise.resolve(angular);
    }
    else if (preset === 'web-components' || preset === 'react') {
        return Promise.resolve(nx);
    }
    else {
        return inquirer
            .prompt([
            {
                name: 'CLI',
                message: "CLI to power the Nx workspace",
                default: 'nx',
                type: 'list',
                choices: [
                    {
                        value: 'nx',
                        name: 'Nx           [Extensible CLI for JavaScript and TypeScript applications]'
                    },
                    {
                        value: 'angular',
                        name: 'Angular CLI  [Extensible CLI for Angular applications]'
                    }
                ]
            }
        ])
            .then(function (a) { return (a.CLI === 'angular' ? angular : nx); });
    }
}
function createSandbox(packageManager, cli) {
    var _a;
    console.log("Creating a sandbox with Nx...");
    var tmpDir = tmp_1.dirSync().name;
    fs_1.writeFileSync(path.join(tmpDir, 'package.json'), JSON.stringify({
        dependencies: (_a = {
                '@nrwl/workspace': nxVersion
            },
            _a[cli.package] = cli.version,
            _a.typescript = tsVersion,
            _a),
        license: 'MIT'
    }));
    child_process_1.execSync(packageManager + " install --silent", {
        cwd: tmpDir,
        stdio: [0, 1, 2]
    });
    return tmpDir;
}
function createApp(tmpDir, cli, parsedArgs, preset) {
    // creating the app itself
    var args = process.argv
        .slice(2)
        .map(function (a) { return "\"" + a + "\""; })
        .join(' ');
    var presetArg = parsedArgs.preset ? '' : " --preset=" + preset;
    console.log(cli.command + " new " + args + presetArg + " --collection=@nrwl/workspace");
    child_process_1.execSync("\"" + path.join(tmpDir, 'node_modules', '.bin', cli.command) + "\" new " + args + presetArg + " --collection=@nrwl/workspace", {
        stdio: [0, 1, 2]
    });
}
function showNxWarning() {
    try {
        child_process_1.execSync('nx --version', { stdio: ['ignore', 'ignore', 'ignore'] });
    }
    catch (e) {
        // no nx found
        console.log('-----------------------------------------------------------');
        console.log("It looks like you don't have the Nx CLI installed globally.");
        console.log("This means that you might have to use \"yarn nx\" or \"npm nx\" to execute commands in your workspace.");
        console.log("If you want to execute the nx command directly, run \"yarn global add @nrwl/cli\" or \"npm install -g @nrwl/cli\"");
        console.log('-----------------------------------------------------------');
    }
}
function showCliWarning(preset, parsedArgs) {
    if (!parsedArgs.cli) {
        if (preset == 'angular' ||
            preset == 'angular-ivy' ||
            preset == 'full-stack') {
            console.log('Because you selected an Angular-specific preset, we generated an Nx workspace powered by the Angular CLi.');
            console.log("If you want want to power the workspace using a different CLI, you can pass it using '--cli'. Find out more by running 'create-nx-workspace --help'.");
        }
        else if (preset === 'web-components' || preset === 'react') {
            console.log('We generated an Nx workspace powered by the Nx CLi.');
            console.log("If you want want to power the workspace using a different CLI, you can pass it using '--cli'. Find out more by running 'create-nx-workspace --help'.");
        }
    }
}
